<?php

namespace SGalinski\SgNews\Hooks\PageLayoutView;

/***************************************************************
 *  Copyright notice
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Backend\View\PageLayoutView;
use TYPO3\CMS\Backend\View\PageLayoutViewDrawItemHookInterface;
use TYPO3\CMS\Core\Localization\LanguageService;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Generic\Backend;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * Class PluginRenderer
 *
 * @package SGalinski\SgNews\Hook\PagelayoutView\PluginRenderer
 */
class PluginRenderer implements PageLayoutViewDrawItemHookInterface {

	/**
	 * Preprocesses the preview rendering of an sg_news content element plugin.
	 *
	 * @param PageLayoutView $parentObject Calling parent object
	 * @param bool $drawItem Whether to draw the item using the default functionalities
	 * @param string $headerContent Header content
	 * @param string $itemContent Item content
	 * @param array $row Record row of tt_content
	 * @noinspection ReferencingObjectsInspection
	 */
	public function preProcess(
		PageLayoutView &$parentObject, &$drawItem, &$headerContent, &$itemContent, array &$row
	): void {
		switch ($row['list_type']) {
			case 'sgnews_overview':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('Overview');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['main']['lDEF'];

				$templateData = [
					'groupBy' => $pluginConfiguration['settings.groupBy']['vDEF'],
					'enableFilter' => $pluginConfiguration['settings.enableFilter']['vDEF'],
					'newsLimit' => $pluginConfiguration['settings.newsLimit']['vDEF'],
					'onlyNewsWithinThisPageSection' => $pluginConfiguration['settings.onlyNewsWithinThisPageSection']['vDEF'],
					'starttime' => $pluginConfiguration['settings.starttime']['vDEF'],
					'endtime' => $pluginConfiguration['settings.endtime']['vDEF'],
					'sortBy' => $pluginConfiguration['settings.sortBy']['vDEF'],
					'sortDirection' => $pluginConfiguration['settings.sortDirection']['vDEF']
				];

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			case 'sgnews_latest':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('Latest');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['main']['lDEF'];

				$categories = $pluginConfiguration['settings.categories']['vDEF'];
				$tags = $pluginConfiguration['settings.tags']['vDEF'];
				$templateData = [
					'limit' => $pluginConfiguration['settings.limit']['vDEF'],
					'categories' => is_string($categories) ? $this->addFieldContentsToRecordIdList(
						'pages', $categories
					) : '',
					'tags' => is_string($tags) ? $this->addFieldContentsToRecordIdList(
						'sys_category', $tags
					) : '',
					'starttime' => $pluginConfiguration['settings.starttime']['vDEF'],
					'endtime' => $pluginConfiguration['settings.endtime']['vDEF'],
					'sortBy' => $pluginConfiguration['settings.sortBy']['vDEF']
				];

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			case 'sgnews_listbycategory':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('ListByCategory');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['main']['lDEF'];

				$categories = $pluginConfiguration['settings.categories']['vDEF'];
				$tags = $pluginConfiguration['settings.tags']['vDEF'];
				$templateData = [
					'newsLimitPerPage' => $pluginConfiguration['settings.newsLimitPerPage']['vDEF'],
					'categories' => is_string($categories) ? $this->addFieldContentsToRecordIdList(
						'pages', $categories
					) : '',
					'tags' => is_string($tags) ? $this->addFieldContentsToRecordIdList('sys_category', $tags) : '',
					'starttime' => $pluginConfiguration['settings.starttime']['vDEF'],
					'endtime' => $pluginConfiguration['settings.endtime']['vDEF'],
					'sortBy' => $pluginConfiguration['settings.sortBy']['vDEF'],
					'sortDirection' => $pluginConfiguration['settings.sortDirection']['vDEF']
				];

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			case 'sgnews_newsbyauthor':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('NewsByAuthor');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['main']['lDEF'];

				$newsAuthors = $pluginConfiguration['settings.newsAuthors']['vDEF'];
				$templateData = [
					'showDetails' => $pluginConfiguration['settings.showDetails']['vDEF'],
					'newsAuthors' => is_string($newsAuthors) ? $this->addFieldContentsToRecordIdList(
						'tx_sgnews_domain_model_author',$newsAuthors,'name'
					) : ''
				];

				// Not using addFieldContentsToRecordIdList to avoid repetitive imploding and exploding.
				$excludedNewsIds = GeneralUtility::intExplode(
					',', $pluginConfiguration['settings.excludedNews']['vDEF'], TRUE
				);
				$excludedNewsListWithTitles = [];
				foreach ($excludedNewsIds as $excludedNewsId) {
					$excludedNewsListWithTitles[] = BackendUtility::getRecord(
							'pages', $excludedNewsId, 'title'
						)['title'] . ' [' . $excludedNewsId . ']';
				}
				$templateData['excludedNews'] = $excludedNewsListWithTitles;

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			default:
				// No need to do anything
		}
	}

	/**
	 * Creates a new StandaloneView object with template and partial rot paths,
	 * attaches the template with the given name to it and returns it.
	 *
	 * @param string $templateName
	 * @return StandaloneView
	 */
	protected
	function createViewWithTemplate(string $templateName): StandaloneView {
		$view = GeneralUtility::makeInstance(StandaloneView::class);
		$view->setTemplateRootPaths(['EXT:sg_news/Resources/Private/Templates/Backend']);
		$view->setPartialRootPaths(['EXT:sg_news/Resources/Private/Partials/Backend']);
		if (!str_ends_with($templateName, '.html')) {
			$templateName .= '.html';
		}
		$view->setTemplate($templateName);
		return $view;
	}

	/**
	 * Adapts the given $headerContent.
	 * To be used in all plugin previews so the Header Contents appear similarly.
	 *
	 * @param $headerContent
	 * @param $row
	 */
	protected
	function adaptPluginHeaderContent(&$headerContent, $row): void {
		$headerContent = '<h4>' . $this->getPluginNameForHeaderContent(
				(int) $row['pid'], $row['list_type']
			) . $headerContent . '</h4>';
	}

	/**
	 * Finds the label of the given $listType element on the page with the given $pid
	 * and returns it wrapped for use in the backend preview's header content.
	 *
	 * @param int $pid
	 * @param string $listType
	 * @return string
	 */
	protected
	function getPluginNameForHeaderContent(int $pid, string $listType): string {
		$languageService = GeneralUtility::makeInstance(LanguageService::class);

		$pluginName = $languageService->sL(
			BackendUtility::getLabelFromItemListMerged(
				$pid, 'tt_content', 'list_type', $listType
			)
		);
		return '<span class="label label-primary">' . $pluginName . '</span>&nbsp;';
	}

	/**
	 * Takes a comma-separated list of record IDs, the corresponding table and optionally the field to look up.
	 * Returns another comma-space-separated list of the same records with the content of the field added.
	 * The returned list should look nice enough to be rendered in the backend preview directly.
	 *
	 * @param string $table
	 * @param string $recordIdList
	 * @param string $field
	 * @return string
	 */
	protected function addFieldContentsToRecordIdList(
		string $table, string $recordIdList, string $field = 'title'
	): string {
		$recordIdsArray = GeneralUtility::intExplode(',', $recordIdList, TRUE);
		$recordsWithTitlesArray = [];

		foreach ($recordIdsArray as $recordId) {
			$recordsWithTitlesArray[] = BackendUtility::getRecord(
					$table, $recordId, $field
				)[$field] . ' [' . $recordId . ']';
		}
		return implode(', ', $recordsWithTitlesArray);
	}
}
